"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var react_1 = __importStar(require("react"));
var enums_1 = require("../../enums");
var DataUtils_1 = require("../../Utils/DataUtils");
var EffectUtils_1 = require("../../Utils/EffectUtils");
var Validation_1 = require("../../Utils/Validation");
var CellEditorDataType_1 = __importDefault(require("../CellEditorDataType/CellEditorDataType"));
var CellEditorValidationMessage_1 = __importDefault(require("../CellEditorValidationMessage/CellEditorValidationMessage"));
var CellEditorState = function (props) {
    var column = props.column, rowData = props.rowData, rowKeyField = props.rowKeyField, dispatch = props.dispatch;
    var _a = react_1.useState(rowData), rowDataState = _a[0], changeValue = _a[1];
    var validationValue = Validation_1.getValidationValue(rowDataState, column);
    var onValueStateChange = function (rowValue) {
        changeValue(rowValue);
    };
    var close = react_1.useCallback(function () {
        var cell = { columnKey: column.key, rowKey: rowData[rowKeyField] };
        dispatch(enums_1.ActionType.CloseEditor, { cell: cell });
    }, [dispatch, column, rowData, rowKeyField]);
    var closeHandler = react_1.useCallback(function () {
        if (!validationValue) {
            var newValue = DataUtils_1.getValueByColumn(rowDataState, column);
            if (DataUtils_1.getValueByColumn(rowData, column) !== newValue) {
                dispatch(enums_1.ActionType.ChangeRowData, { newValue: DataUtils_1.replaceValue(rowData, column, newValue) });
            }
            close();
        }
    }, [validationValue, dispatch, close, rowDataState, column, rowData]);
    react_1.useEffect(function () {
        return EffectUtils_1.addEscEnterKeyEffect(close, closeHandler);
    }, [close, closeHandler]);
    var dispatchHandler = function (action, actionData) {
        if (action === enums_1.ActionType.CloseEditor) {
            closeHandler();
        }
        else if (action === enums_1.ActionType.ChangeRowData) {
            onValueStateChange(actionData.newValue);
        }
        else {
            dispatch(action, actionData);
        }
    };
    var stateProps = __assign(__assign({}, props), {
        dispatch: dispatchHandler,
        rowData: rowDataState,
        value: DataUtils_1.getValueByColumn(rowDataState, column)
    });
    return (react_1["default"].createElement(react_1["default"].Fragment, null,
        react_1["default"].createElement("div", { className: "ka-cell-editor " + (validationValue ? 'ka-cell-editor-validation-error' : '') },
            react_1["default"].createElement(CellEditorDataType_1["default"], __assign({}, stateProps)),
            validationValue && react_1["default"].createElement(CellEditorValidationMessage_1["default"], { message: validationValue }))));
};
exports["default"] = CellEditorState;
