"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
var enums_1 = require("../enums");
var ArrayUtils_1 = require("./ArrayUtils");
var CellUtils_1 = require("./CellUtils");
var GroupUtils_1 = require("./GroupUtils");
var HeadRowUtils_1 = require("./HeadRowUtils");
exports.wrapDispatch = function (tableProps, theadRef) {
    var columns = tableProps.columns, data = tableProps.data, _a = tableProps.editableCells, editableCells = _a === void 0 ? [] : _a, _b = tableProps.groupsExpanded, groupsExpanded = _b === void 0 ? [] : _b, _c = tableProps.onDataChange, onDataChange = _c === void 0 ? function () { } : _c, _d = tableProps.onEvent, onEvent = _d === void 0 ? function () { } : _d, onOptionChange = tableProps.onOptionChange, rowKeyField = tableProps.rowKeyField, _e = tableProps.selectedRows, selectedRows = _e === void 0 ? [] : _e, virtualScrolling = tableProps.virtualScrolling;
    return function (action, actionData) {
        var _a;
        switch (action) {
            case enums_1.ActionType.OpenEditor: {
                var newEditableCells = CellUtils_1.addItemToEditableCells(actionData.cell, editableCells);
                onOptionChange({ editableCells: newEditableCells });
                break;
            }
            case enums_1.ActionType.CloseEditor: {
                var newEditableCells = CellUtils_1.removeItemFromEditableCells(actionData.cell, editableCells);
                onOptionChange({ editableCells: newEditableCells });
                break;
            }
            case enums_1.ActionType.ChangeFilterRow:
                var newColumns = ArrayUtils_1.getCopyOfArrayAndInsertOrReplaceItem(actionData.column, 'key', columns);
                onOptionChange({ columns: newColumns });
                break;
            case enums_1.ActionType.ChangeRowData:
                var newData = ArrayUtils_1.getCopyOfArrayAndInsertOrReplaceItem(actionData.newValue, rowKeyField, data);
                onDataChange(newData);
                break;
            case enums_1.ActionType.SelectRow:
                onOptionChange({ selectedRows: __spreadArrays(selectedRows, [actionData.rowKeyValue]) });
                break;
            case enums_1.ActionType.DeselectRowData:
                onOptionChange({ selectedRows: __spreadArrays(selectedRows).filter(function (s) { return s !== actionData.rowKeyValue; }) });
                break;
            case enums_1.ActionType.ChangeSorting:
                var sortedColumns = HeadRowUtils_1.getSortedColumns(columns, actionData.column);
                onOptionChange({ columns: sortedColumns });
                break;
            case enums_1.ActionType.ChangeVirtualScrollingHeightSettings:
                onOptionChange(actionData);
                break;
            case enums_1.ActionType.ScrollTable:
                (_a = theadRef.current) === null || _a === void 0 ? void 0 : _a.scrollTo({ left: actionData.scrollLeft });
                if (virtualScrolling) {
                    var scrollPosition = actionData.scrollTop;
                    if (virtualScrolling) {
                        onOptionChange({ virtualScrolling: __assign(__assign({}, virtualScrolling), { scrollPosition: scrollPosition }) });
                    }
                }
                break;
            case enums_1.ActionType.UpdateGroupsExpanded:
                var newGroupsExpanded = GroupUtils_1.updateExpandedGroups(groupsExpanded, actionData.groupKey);
                actionData.newValue = { groupsExpanded: newGroupsExpanded }; // BC
                onOptionChange({ groupsExpanded: newGroupsExpanded });
                break;
        }
        onEvent(action, actionData);
    };
};
