"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var defaultOptions_1 = __importDefault(require("../defaultOptions"));
var enums_1 = require("../enums");
var CommonUtils_1 = require("./CommonUtils");
var DataUtils_1 = require("./DataUtils");
exports.getRowEditableCells = function (rowKeyValue, editableCells) {
    return editableCells.filter(function (c) { return c.rowKey === rowKeyValue; });
};
exports.searchData = function (columns, data, searchText) {
    return columns.reduce(function (initialData, c) {
        var filterFunction = function (item) {
            if (c.search) {
                return c.search(searchText, item, c);
            }
            if (initialData.indexOf(item) >= 0) {
                return false;
            }
            var columnValue = DataUtils_1.getValueByColumn(item, c);
            if (columnValue == null) {
                return false;
            }
            return columnValue.toString().toLowerCase().includes(searchText.toLowerCase());
        };
        return initialData.concat(data.filter(filterFunction));
    }, []);
};
exports.filterData = function (data, columns) {
    return columns.reduce(function (initialData, column) {
        if (CommonUtils_1.isEmpty(column.filterRowValue)
            && column.filterRowOperator !== enums_1.FilterOperatorName.IsEmpty
            && column.filterRowOperator !== enums_1.FilterOperatorName.IsNotEmpty) {
            return initialData;
        }
        var filterRowOperator = column.filterRowOperator
            || exports.getDefaultOperatorForType(column.dataType || defaultOptions_1["default"].columnDataType);
        var filterOperator = exports.predefinedFilterOperators.find(function (fo) { return filterRowOperator === fo.name; });
        if (!filterOperator) {
            throw new Error("'" + column.filterRowOperator + "' has not found in predefinedFilterOperators array, available operators: " + exports.predefinedFilterOperators.map(function (o) { return o.name; }).join(', '));
        }
        var compare = filterOperator.compare;
        return initialData.filter(function (d) {
            var fieldValue = DataUtils_1.getValueByColumn(d, column);
            var conditionValue = column.filterRowValue;
            if (column.dataType === enums_1.DataType.Date) {
                fieldValue = fieldValue == null ? fieldValue : new Date(fieldValue).setHours(0, 0, 0, 0);
                conditionValue = conditionValue == null ? conditionValue : new Date(conditionValue).setHours(0, 0, 0, 0);
            }
            return compare(fieldValue, conditionValue);
        });
    }, data);
};
exports.getDefaultOperatorForType = function (type) {
    var filterOperator = exports.predefinedFilterOperators.find(function (o) { return o.defaultForTypes && o.defaultForTypes.includes(type); });
    return (filterOperator && filterOperator.name) || '=';
};
exports.predefinedFilterOperators = [{
        compare: function (fieldValue, conditionValue) {
            return fieldValue === conditionValue;
        },
        defaultForTypes: [enums_1.DataType.Boolean, enums_1.DataType.Number, enums_1.DataType.Date],
        name: enums_1.FilterOperatorName.Equal
    }, {
        compare: function (fieldValue, conditionValue) {
            return fieldValue > conditionValue;
        },
        name: enums_1.FilterOperatorName.MoreThan
    }, {
        compare: function (fieldValue, conditionValue) {
            return fieldValue < conditionValue;
        },
        name: enums_1.FilterOperatorName.LessThan
    }, {
        compare: function (fieldValue, conditionValue) {
            return fieldValue >= conditionValue;
        },
        name: enums_1.FilterOperatorName.MoreThanOrEqual
    }, {
        compare: function (fieldValue, conditionValue) {
            return fieldValue <= conditionValue;
        },
        name: enums_1.FilterOperatorName.LessThanOrEqual
    }, {
        compare: function (fieldValue, conditionValue) {
            return fieldValue != null && fieldValue.toString().toLowerCase().includes(conditionValue.toString().toLowerCase());
        },
        defaultForTypes: [enums_1.DataType.String],
        name: enums_1.FilterOperatorName.Contains
    }, {
        compare: function (fieldValue) {
            return CommonUtils_1.isEmpty(fieldValue);
        },
        name: enums_1.FilterOperatorName.IsEmpty
    }, {
        compare: function (fieldValue) {
            return !CommonUtils_1.isEmpty(fieldValue);
        },
        name: enums_1.FilterOperatorName.IsNotEmpty
    }];
